--[[
SunkenFortress By Crinos512
  Based off of "Bezier thing" by wlourf (26 Feb. 2010) http://u-scripts.blogspot.com/
           and "Ring Meters" by londonali1010 (2009)
]]

require 'cairo'
require 'math'

function draw_ring(cr,t,pt)
	local w,h=conky_window.width,conky_window.height

	local xc,yc,ring_r,ring_w,sa,ea=pt['x'],pt['y'],pt['radius'],pt['thickness'],pt['start_angle'],pt['end_angle']
	local bgc, bga, fgc, fga=pt['bg_colour'], pt['bg_alpha'], pt['fg_colour'], pt['fg_alpha']

	local angle_0=sa*(2*math.pi/360)-math.pi/2
	local angle_f=ea*(2*math.pi/360)-math.pi/2
	local t_arc=t*(angle_f-angle_0)

	local wn, dn=pt['warning'], pt['danger']
	local cc, cd, wa, wc, da, dc=pt['colour_coding'], pt['coding_direction'], pt['warning_alpha'], pt['warning_colour'], pt['danger_alpha'], pt['danger_colour']

	local str=''
	local value=0

	str=string.format('${%s %s}',pt['name'],pt['arg'])
	str=conky_parse(str)
	value=tonumber(str)

	-- Draw background ring
	cairo_arc(cr,xc,yc,ring_r,angle_0,angle_f)
	cairo_set_line_width(cr,ring_w)
	cairo_set_source_rgba(cr,rgb_to_r_g_b(bgc,bga))
	cairo_stroke(cr)

	-- Draw indicator ring
	cairo_arc(cr,xc,yc,ring_r,angle_0,angle_0+t_arc)
	if cc == false then
		cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
	else
		if cd == true then
		if value==nil then value=0 end
			if value >= wn and value < dn then
				cairo_set_source_rgba(cr,rgb_to_r_g_b(wc,wa))
			elseif value >= dn then
				cairo_set_source_rgba(cr,rgb_to_r_g_b(dc,da))
			else
				cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
			end
		else
			if value <= dn then
				cairo_set_source_rgba(cr,rgb_to_r_g_b(dc,da))
			elseif value <= wn then
				cairo_set_source_rgba(cr,rgb_to_r_g_b(wc,wa))
			else
				cairo_set_source_rgba(cr,rgb_to_r_g_b(fgc,fga))
			end
		end
	end
	cairo_stroke(cr)
end

function draw_edge(t,show)
--draw lines of the shape
	if show then
		cairo_set_line_width(cr,2)
		cairo_set_source_rgba (cr,1, 0,0,1)
		cairo_curve_to(cr,t[1],t[2],t[3],t[4],t[5],t[6])
		cairo_stroke(cr)
	end
end

function calcul_shape (t,show)
--calculation of dots for bezier curves
	tx,ty,tcx,tcy,tr=t[1],t[2],t[3],t[4],t[5]
	if tx==tcx then tcx=tcx+0.000000001 end

    if tx~=tcx then
	    a0=(ty-tcy)/(tx-tcx)
	    b0=tcy-a0*tcx
	end

    angle = math.atan(a0)

    r0=tr
   	r1= math.sqrt(math.pow(tx-tcx,2)+math.pow(ty-tcy,2))
	gamma = pi/2-math.atan(r0/r1)

 	local delta=pi/2
 	if tx<tcx then delta=-pi/2 end

	angle0=angle-gamma+delta
	angle1=angle+gamma+delta

	tx0 = tcx + r0*math.sin(angle0)
	ty0 = tcy - r0*math.cos(angle0)
	tx1 = tcx + r0*math.sin(angle1)
	ty1 = tcy - r0*math.cos(angle1)

	if show then
		cairo_set_line_width (cr,1)
		if tx==tcx then
			y0,y1=ty,tcy
			x0,x1=tx,tcx
		else
			x0=tx
			y0=a0*x0+b0
			x1=tcx
			y1=a0*x1+b0
		end

		cairo_set_source_rgba (cr,0, 0,1,0.5)
		cairo_move_to(cr,x0,y0)
		cairo_line_to(cr,x1,y1)
		cairo_stroke(cr)
		--P
		cairo_arc(cr,tx,ty,2,0,2*pi)
		cairo_stroke(cr)
		--Pc
		cairo_arc(cr,tcx,tcy,1,0,2*pi)
		cairo_stroke(cr)
		--Circle
		cairo_set_source_rgba (cr,1, 0, 0,.5)
		cairo_arc(cr,tcx,tcy,tr,0,2*pi)
		cairo_stroke(cr)
		--points tangeants
		cairo_set_source_rgba (cr,1, 0, 1,0.5)
		cairo_arc(cr,tx0,ty0,1,0,2*pi)
		cairo_stroke(cr)
		cairo_move_to(cr,tx0,ty0)
		cairo_line_to(cr,tx,ty)
		cairo_stroke(cr)

		cairo_set_source_rgba (cr,0, 1, 1,0.5)
		cairo_arc(cr,tx1,ty1,1,0,2*pi)
		cairo_stroke(cr)
		cairo_move_to(cr,tx1,ty1)
		cairo_line_to(cr,tx,ty)
		cairo_stroke(cr)
	end

    if tx==tcx then tx=tcx*1.0001 end

    return tx0,ty0, tx1,ty1
end

function intercept(p11,p12,p21,p22,show)
--calculation of interscetion of two lines
    a1=(p12[2]-p11[2])/(p12[1]-p11[1])
    a2=(p22[2]-p21[2])/(p22[1]-p21[1])
    b1=p11[2]-a1*p11[1]
    b2=p21[2]-a2*p21[1]
    X=(b2-b1)/(a1-a2)
    Y=a1*X+b1

	if show then
		cairo_set_source_rgba (cr,0, 0.5, 0, 0.25)
		q0x1=0
		q0x2=cwidth
		q0y1=q0x1*a1+b1
		q0y2=q0x2*a1+b1
		cairo_move_to(cr,q0x1,q0y1)
		cairo_line_to(cr,q0x2,q0y2)
		cairo_stroke(cr)
		q0y1=q0x1*a2+b2
		q0y2=q0x2*a2+b2
		cairo_move_to(cr,q0x1,q0y1)
		cairo_line_to(cr,q0x2,q0y2)
		cairo_stroke(cr)

		cairo_set_source_rgba (cr,0.5, 0.5, 0.5,1)
		cairo_arc(cr,X,Y,2,0,2*pi)
		cairo_fill(cr)
    end

    return {X,Y}
end

function rgb_to_r_g_b(colour,alpha)
	return ((colour / 0x10000) % 0x100) / 255., ((colour / 0x100) % 0x100) / 255., (colour % 0x100) / 255.,alpha
end


--Settings
function twinkle()
	local r0=3
	local hg=5 --magnify factor
	local xc,yc=cwidth/2,cheight/2
	local dd=200
	local points = {
	--	Px			Py,			Cx,		Cy,		Radius,		Start side
		{xc,			yc-dd-hg*cpu0,		xc,		yc,		r0,		0},		--tm
		{xc+dd+hg*cpu1,		yc,			xc,		yc,		r0,		0},		--cr
		{xc,			yc+dd+hg*cpu2,		xc,		yc,		r0,		0},		--bm
		{xc-dd-hg*cpu3,		yc,			xc,		yc,		r0,		0},		--cl
	}
	return points
end

function rings()
  local settings_table = {
    {
	name='cpu',			arg='',				max=1,
	bg_colour=0x000000,		bg_alpha=0.02,
	fg_colour=0xFFFFFF,		fg_alpha=0.02,
	x=cwidth/2, y=cheight/2,	radius=cpu*2,			thickness=3,
	start_angle=0,			end_angle=360,
	colour_coding=true,		coding_direction=true,
	warning=25,			warning_colour=0xFFFFFF,	warning_alpha=0.02,
	danger=75,			danger_colour=0xFFFFFF,		danger_alpha=0.02
    },
    {
	name='memperc',			arg='',				max=1,
	bg_colour=0x000000,		bg_alpha=0.02,
	fg_colour=0xFFFFFF,		fg_alpha=0.02,
	x=cwidth/2, y=cheight/2,	radius=260+(memperc/2),		thickness=memperc,
	start_angle=0,			end_angle=360,
	colour_coding=true,		coding_direction=true,
	warning=25,			warning_colour=0xFFFFFF,	warning_alpha=0.02,
	danger=75,			danger_colour=0xFFFFFF,		danger_alpha=0.02
    },
    {
	name='swapperc',		arg='',				max=1,
	bg_colour=0x000000,		bg_alpha=0.01,
	fg_colour=0xFF0000,		fg_alpha=0.05,
	x=cwidth/2, y=cheight/2,	radius=260+(swapperc/2),	thickness=swapperc,
	start_angle=0,			end_angle=360,
	colour_coding=true,		coding_direction=true,
	warning=25,			warning_colour=0xFF0000,	warning_alpha=0.05,
	danger=75,			danger_colour=0xFF0000,		danger_alpha=0.05
    },
    {
	name='fs_free_perc',		arg='/home',			max=100,
	bg_colour=0x000000,		bg_alpha=0.1,
	fg_colour=0x0000FF,		fg_alpha=0.1,
	x=cwidth/2, y=cheight/2,	radius=248,			thickness=5,
	start_angle=0,			end_angle=360,
	colour_coding=true,		coding_direction=true,
	warning=80,			warning_colour=0x0000FF,	warning_alpha=0.2,
	danger=90,			danger_colour=0x0000FF,		danger_alpha=0.3
    },
    {
	name='fs_free_perc',		arg='/',			max=100,
	bg_colour=0x000000,		bg_alpha=0.1,
	fg_colour=0x0000FF,		fg_alpha=0.1,
	x=cwidth/2, y=cheight/2,	radius=255,			thickness=5,
	start_angle=0,			end_angle=360,
	colour_coding=true,		coding_direction=true,
	warning=80,			warning_colour=0x0000FF,	warning_alpha=0.2,
	danger=90,			danger_colour=0x0000FF,		danger_alpha=0.3
    },
  }
  return settings_table
end


--the main function called by conky
function conky_main(int_show,offset_x,offset_y,black_background)
	if conky_window==nil then return end

	--set some variables here to fit with your configuration
	local updates=tonumber(conky_parse("${updates}"))
	local color1=0xFFFFFF
	local color2=0xFFFFFF
	local alpha1=0
	local alpha2=.5
	local light_center_x,light_center_y --are set below


	--globals variables which can be used
	cwidth,cheight=conky_window.width,conky_window.height
	rad = 6 --default radius for the variables
	cpu0,cpu1,cpu2,cpu3,cpu=rad,rad,rad,rad,rad
	memperc,swapperc=rad,rad
--	running,mixer=rad,rad
	if updates >5 then
		cpu =math.max(tonumber(conky_parse("${cpu}")),rad)
		cpu0=math.max(tonumber(conky_parse("${cpu cpu0}")),rad)
		cpu1=math.max(tonumber(conky_parse("${cpu cpu1}")),rad)
		cpu2=math.max(tonumber(conky_parse("${cpu cpu2}")),rad)
		cpu3=math.max(tonumber(conky_parse("${cpu cpu3}")),rad)
--		running=math.max(tonumber(conky_parse("${running_processes}")),rad)
--		mixer=math.max(tonumber(conky_parse("${mixer}"))/655.36,rad)
		memperc=math.max(tonumber(conky_parse("${memperc}")),rad)
		swapperc=math.max(tonumber(conky_parse("${swapperc}")),rad)
	end


	--------------------end of variables-------------------------
	pi=math.pi
	local bShow=(int_show=="1")
	local cs=cairo_xlib_surface_create(conky_window.display, conky_window.drawable, conky_window.visual, cwidth, cheight)
	cr=cairo_create(cs)
	local function setup_rings(cr,pt)
		local str=''
		local value=0

		str=string.format('${%s %s}',pt['name'],pt['arg'])
		str=conky_parse(str)
		value=tonumber(str)
		if value == nil then value = 0 end
		pct=value/pt['max']
		draw_ring(cr,pct,pt)
	end

	if black_background=="1" then
		cairo_set_source_rgba (cr,0, 0, 0,.5)
		cairo_rectangle (cr,0,0,cwidth,cheight)
		cairo_fill(cr)
	end

	local bands = rings()
	for i in pairs(bands) do
		setup_rings(cr,bands[i])
	end

	local points = twinkle()
	light_center_x,light_center_y=cwidth/2,cheight/2
	arrayp={}
	--calculate the shape
	for i,tab in ipairs(points) do
		if tab[6]==0 then
			p1x,p1y,p0x,p0y = calcul_shape(tab,bShow)
		else
			p0x,p0y,p1x,p1y = calcul_shape(tab,bShow)
		end
		if i==1 then
			table.insert(arrayp,{p0x,p0y,tab[1],tab[2],p1x,p1y})
			draw_edge({p0x,p0y,tab[1],tab[2],p1x,p1y},bShow)
		end
		pt={p0x,p0y}
		invpt={p1x,p1y}

		if i>1 then
			pointe=intercept(previousPoint,previousStart,
						{tab[1],tab[2]},pt,bShow)
			draw_edge({previousStart[1],previousStart[2],pointe[1],pointe[2],pt[1],pt[2]},bShow)
			table.insert(arrayp,{previousStart[1],previousStart[2],pointe[1],pointe[2],pt[1],pt[2]})

			table.insert(arrayp,{p0x,p0y,tab[1],tab[2],p1x,p1y})
			draw_edge({p0x,p0y,tab[1],tab[2],p1x,p1y},bShow)

			previousPoint={tab[1],tab[2]}
			previousStart=invpt
		else
			firstPoint={tab[1],tab[2]}
			firstStart=pt
			previousPoint={tab[1],tab[2]}
			previousStart=invpt
		end

		if i == #points then
			pointe=intercept(firstPoint,firstStart,
						{tab[1],tab[2]},invpt,bShow)
			table.insert(arrayp,{previousStart[1],previousStart[2],pointe[1],pointe[2],firstStart[1],firstStart[2]})
			draw_edge({previousStart[1],previousStart[2],pointe[1],pointe[2],firstStart[1],firstStart[2]},bShow)
		end
	end

	cairo_fill_preserve (cr);
	cairo_set_line_width(cr,4)
	cairo_stroke(cr)

	--draw shape
	cairo_translate(cr,offset_x,offset_y)
	cairo_move_to (cr, arrayp[1][1],arrayp[1][2])
	for i =1,#arrayp do
		cairo_curve_to(cr,arrayp[i][1],arrayp[i][2],arrayp[i][3],arrayp[i][4],arrayp[i][5],arrayp[i][6])
	end
	cairo_close_path (cr);

	--colorize the shape
	local s=cwidth/2
	pat = cairo_pattern_create_radial (light_center_x, light_center_y, 1,
                               			light_center_x,light_center_y,0.8*s)
	cairo_pattern_add_color_stop_rgba (pat, 0, rgb_to_r_g_b(color1,alpha1))
	cairo_pattern_add_color_stop_rgba (pat, 1, rgb_to_r_g_b(color2,alpha2))
	cairo_set_source (cr, pat)
	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_WINDING);
	cairo_fill_preserve(cr)

	cairo_destroy(cr)
	cairo_surface_destroy(cs)
end
